#ifndef HRTIMER_X86_H__
#define HRTIMER_X86_H__

#include <sys/cdefs.h>
#include <stdint.h>

/*
 * Notes:
 * It would have been nice to use inline or __inline__ for this,
 * but in their infinite wisdom, the standards implementors and gcc
 * don't seem to agree on semantics, so the macro is just more portable.
 *
 * gcc on x86_64 seems to interpret =A as RAX rather than the EAX/EDX
 * pair.  Not sure if the =a/=d form works on i686, perhaps that is the
 * portable way to go?
 */

__BEGIN_DECLS

/* returns 64-bit Intel x86 (X86) Time Stamp Counter. */
#if __WORDSIZE == 64
# define HRTIMER_X86_GET(val) do { \
     unsigned int a, d; \
     __asm__ __volatile__("rdtsc" : "=a" (a), "=d" (d)); \
     (val) = ((uint64_t) a) | (((uint64_t)d)<<32); \
} while(0)
#else /* __WORDSIZE != 64 */
# define HRTIMER_X86_GET(val)  __asm__ __volatile__("rdtsc" : "=A" (val) : )
#endif /* __WORDSIZE */

/* returns 64-bit Intel x86 (X86) Time Stamp Counter. */
extern uint64_t hrtimer_x86(void);

/* returns the number of cycles per microsecond */
extern uint64_t hrtimer_x86_calibrate(void);

/* turns on debugging if the argument is 1, returns the previous debug value */
extern int hrtimer_debug(int);
__END_DECLS

#endif /* HRTIMER_X86_H__ */
